@@ -25,39 +25,13 @@
 
 #include "gf-logical-monitor-config-private.h"
 #include "gf-monitor-config-private.h"
+#include "gf-monitor-config-utils.h"
 #include "gf-monitor-spec-private.h"
 #include "gf-monitors-config-private.h"
 #include "gf-rectangle-private.h"
 
 G_DEFINE_TYPE (GfMonitorsConfig, gf_monitors_config, G_TYPE_OBJECT)
 
-static gboolean
-has_adjacent_neighbour (GfMonitorsConfig       *config,
-                        GfLogicalMonitorConfig *logical_monitor_config)
-{
-  GList *l;
-
-  if (!config->logical_monitor_configs->next)
-    {
-      g_assert (config->logical_monitor_configs->data == logical_monitor_config);
-      return TRUE;
-    }
-
-  for (l = config->logical_monitor_configs; l; l = l->next)
-    {
-      GfLogicalMonitorConfig *other_logical_monitor_config = l->data;
-
-      if (logical_monitor_config == other_logical_monitor_config)
-        continue;
-
-      if (gf_rectangle_is_adjacent_to (&logical_monitor_config->layout,
-                                       &other_logical_monitor_config->layout))
-        return TRUE;
-    }
-
-  return FALSE;
-}
-
 static gboolean
 gf_monitors_config_is_monitor_enabled (GfMonitorsConfig *config,
                                        GfMonitorSpec    *monitor_spec)
@@ -67,8 +41,9 @@ gf_monitors_config_is_monitor_enabled (GfMonitorsConfig *config,
 }
 
 static GfMonitorsConfigKey *
-gf_monitors_config_key_new (GList *logical_monitor_configs,
-                            GList *disabled_monitor_specs)
+gf_monitors_config_key_new (GList                      *logical_monitor_configs,
+                            GList                      *disabled_monitor_specs,
+                            GfLogicalMonitorLayoutMode  layout_mode)
 {
   GfMonitorsConfigKey *config_key;
   GList *monitor_specs;
@@ -102,6 +77,7 @@ gf_monitors_config_key_new (GList *logical_monitor_configs,
 
   config_key = g_new0 (GfMonitorsConfigKey, 1);
   config_key->monitor_specs = monitor_specs;
+  config_key->layout_mode = layout_mode;
 
   return config_key;
 }
@@ -150,9 +126,10 @@ gf_monitors_config_new_full (GList                      *logical_monitor_configs
   config = g_object_new (GF_TYPE_MONITORS_CONFIG, NULL);
   config->logical_monitor_configs = logical_monitor_configs;
   config->disabled_monitor_specs = disabled_monitor_specs;
-  config->key = gf_monitors_config_key_new (logical_monitor_configs,
-                                            disabled_monitor_specs);
   config->layout_mode = layout_mode;
+  config->key = gf_monitors_config_key_new (logical_monitor_configs,
+                                            disabled_monitor_specs,
+                                            layout_mode);
   config->flags = flags;
   config->switch_config = GF_MONITOR_SWITCH_CONFIG_UNKNOWN;
 
@@ -224,7 +201,7 @@ gf_monitors_config_key_hash (gconstpointer data)
   GList *l;
 
   config_key = data;
-  hash = 0;
+  hash = config_key->layout_mode;
 
   for (l = config_key->monitor_specs; l; l = l->next)
     {
@@ -250,6 +227,9 @@ gf_monitors_config_key_equal (gconstpointer data_a,
   config_key_a = data_a;
   config_key_b = data_b;
 
+  if (config_key_a->layout_mode != config_key_b->layout_mode)
+    return FALSE;
+
   for (l_a = config_key_a->monitor_specs, l_b = config_key_b->monitor_specs;
        l_a && l_b;
        l_a = l_a->next, l_b = l_b->next)
@@ -279,85 +259,13 @@ gf_verify_monitors_config (GfMonitorsConfig  *config,
                            GfMonitorManager  *monitor_manager,
                            GError           **error)
 {
-  gint min_x, min_y;
-  gboolean has_primary;
-  GList *region;
   GList *l;
-  gboolean global_scale_required;
 
-  if (!config->logical_monitor_configs)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "Monitors config incomplete");
-
-      return FALSE;
-    }
-
-  global_scale_required = !!(gf_monitor_manager_get_capabilities (monitor_manager) &
-                             GF_MONITOR_MANAGER_CAPABILITY_GLOBAL_SCALE_REQUIRED);
-
-  min_x = INT_MAX;
-  min_y = INT_MAX;
-  region = NULL;
-  has_primary = FALSE;
-  for (l = config->logical_monitor_configs; l; l = l->next)
-    {
-      GfLogicalMonitorConfig *logical_monitor_config = l->data;
-
-      if (global_scale_required)
-        {
-          GfLogicalMonitorConfig *prev_logical_monitor_config;
-
-          prev_logical_monitor_config = l->prev ? l->prev->data : NULL;
-
-          if (prev_logical_monitor_config &&
-              (prev_logical_monitor_config->scale !=
-               logical_monitor_config->scale))
-            {
-              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           "Logical monitor scales must be identical");
-
-              return FALSE;
-            }
-        }
-
-      if (gf_rectangle_overlaps_with_region (region, &logical_monitor_config->layout))
-        {
-          g_list_free (region);
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       "Logical monitors overlap");
-
-          return FALSE;
-        }
-
-      if (has_primary && logical_monitor_config->is_primary)
-        {
-          g_list_free (region);
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       "Config contains multiple primary logical monitors");
-
-          return FALSE;
-        }
-      else if (logical_monitor_config->is_primary)
-        {
-          has_primary = TRUE;
-        }
-
-      if (!has_adjacent_neighbour (config, logical_monitor_config))
-        {
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       "Logical monitors not adjacent");
-
-          return FALSE;
-        }
-
-      min_x = MIN (logical_monitor_config->layout.x, min_x);
-      min_y = MIN (logical_monitor_config->layout.y, min_y);
-
-      region = g_list_prepend (region, &logical_monitor_config->layout);
-    }
-
-  g_list_free (region);
+  if (!gf_verify_logical_monitor_config_list (config->logical_monitor_configs,
+                                              config->layout_mode,
+                                              monitor_manager,
+                                              error))
+    return FALSE;
 
   for (l = config->disabled_monitor_specs; l; l = l->next)
     {
@@ -371,21 +279,5 @@ gf_verify_monitors_config (GfMonitorsConfig  *config,
         }
     }
 
-  if (min_x != 0 || min_y != 0)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "Logical monitors positions are offset");
-
-      return FALSE;
-    }
-
-  if (!has_primary)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "Config is missing primary logical");
-
-      return FALSE;
-    }
-
   return TRUE;
 }
