@@ -19,34 +19,35 @@
 #include "gf-desktop-window.h"
 
 #include <gdk/gdkx.h>
-#include <libcommon/gf-background-utils.h>
-#include <libgnome-desktop/gnome-bg.h>
 #include <glib/gi18n.h>
 #include <X11/Xatom.h>
 
 #include "gf-background.h"
 #include "gf-icon-view.h"
+#include "libcommon/gf-bg.h"
 
 struct _GfDesktopWindow
 {
-  GtkWindow        parent;
+  GtkWindow         parent;
 
-  gboolean         draw_background;
-  GfBackground    *background;
-  gboolean         event_filter_added;
-  cairo_surface_t *surface;
+  GfMonitorManager *monitor_manager;
 
-  gboolean         show_icons;
-  GtkWidget       *icon_view;
+  gboolean          draw_background;
+  GfBackground     *background;
+  gboolean          event_filter_added;
+  cairo_surface_t  *surface;
 
-  int              width;
-  int              height;
+  gboolean          show_icons;
+  GtkWidget        *icon_view;
 
-  guint            move_resize_id;
+  int               width;
+  int               height;
 
-  gboolean         ready;
+  guint             move_resize_id;
 
-  GdkRGBA         *representative_color;
+  gboolean          ready;
+
+  GdkRGBA          *representative_color;
 };
 
 enum
@@ -209,9 +210,9 @@ ensure_surface (GfDesktopWindow *self)
 
   display = gtk_widget_get_display (widget);
 
-  self->surface = gf_background_surface_get_from_root (display,
-                                                       self->width,
-                                                       self->height);
+  self->surface = gf_bg_get_surface_from_root (display,
+                                               self->width,
+                                               self->height);
 
   gtk_widget_queue_draw (widget);
 }
@@ -255,7 +256,7 @@ filter_func (GdkXEvent *xevent,
           ensure_surface (self);
         }
     }
-  else if (x->xproperty.atom != color_atom)
+  else if (x->xproperty.atom == color_atom)
     {
       update_representative_color (self);
     }
@@ -832,6 +833,13 @@ gf_desktop_window_new (gboolean   draw_background,
   return window;
 }
 
+void
+gf_desktop_window_set_monitor_manager (GfDesktopWindow  *self,
+                                       GfMonitorManager *monitor_manager)
+{
+  self->monitor_manager = monitor_manager;
+}
+
 gboolean
 gf_desktop_window_is_ready (GfDesktopWindow *self)
 {
